/*-----------------------------------------------------------------------------
  Copyright (c) 2015 by Freescale Semiconductor, All Rights reserved.

  This is unpublished proprietary source code of Freescale.
  The copyright notice above does not imply any actual or
  intended publication of such source code.

  FREESCALE CONFIDENTIAL PROPRIETARY
  ---------------------------------------------------------------------------*/
/*-----------------------------------------------------------------------------
  FILE NAME      : memory_update_protocol_wildcard_testflash.c
  DEPARTMENT     : AISG
  AUTHOR         : Pradip Singh
  AUTHOR'S EMAIL : pradip.singh@freescale.com
  -----------------------------------------------------------------------------
  REVISION HISTORY
  AUTHOR       			     Date D/M/Y                DESCRIPTION              
  ---------------------- 	----------              --------------------- 
  Pradip Singh (b09147) 	 27/7/2015               Initial Version
  --------------------- 	 ----------             ---------------------
  KEYWORDS :HSM, Security
  -----------------------------------------------------------------------------
  PURPOSE :  Implement C functions for clock configuration. 
  ---------------------------------------------------------------------------*/

/*=============================================================================
  INCLUDE FILES
  ===========================================================================*/
#include "mem_map.h"
/*=============================================================================
  LOCAL CONSTANTS
  ===========================================================================*/

/*=============================================================================
  LOCAL TYPEDEFS (STRUCTURES, UNIONS, ENUMS)
  ===========================================================================*/
extern struct HSM_CMD_struct HSM_CMD;
/*=============================================================================
  LOCAL VARIABLES
  ===========================================================================*/

/*=============================================================================
  LOCAL FUNCTION PROTOTYPES
  ===========================================================================*/

/*=============================================================================
  GLOBAL VARIABLES
  ===========================================================================*/

/*=============================================================================
  LOCAL FUNCTIONS
  ===========================================================================*/

/*=============================================================================
  FUNCTION: host_pll_fxosc_setup

  DESCRIPTION:
  This function configures the PLL clock for FXOSC reference clock for MPC5746C

  ARGUMENTS PASSED:
  None
  
  RETURN VALUE:
  None
  
  PRE-CONDITIONS:
  None

  POST-CONDITIONS:
  None

  IMPORTANT NOTES:
  None
  ===========================================================================*/
void host_pll_fxosc_setup()
{
    /* select FXOSC as reference for PLL */
    MC_CGM.AC5_SC.B.SELCTL = 1; 					

    /* Configure PLL Dividers - 160MHz from 40Mhz xtal oscillator */
    /* divide by 8 */
    PLLDIG.PLLDV.B.RFDPHI1 = 2;
    /* divide by 4 */
    PLLDIG.PLLDV.B.RFDPHI = 1;
    /* divide by 1 */
    PLLDIG.PLLDV.B.PREDIV = 1;
    /* divide by 16 */
    PLLDIG.PLLDV.B.MFD = 16; 						

    /* fractional loop division factor = 0 */
    PLLDIG.PLLFD.B.MFN = 0; 						

    /* loss of lock interrupt enable */
    PLLDIG.PLLCR.B.LOLIE = 1;
    /* denominator of fractional loop division factor*/
    PLLDIG.PLLCAL3.R = 0x09C3C000;
    /* sigma delta modulation disabled */
    PLLDIG.PLLFD.B.SMDEN = 1; 						

    /* crystal on */
    MC_ME.DRUN_MC.B.FXOSCON=1;
    /* Enable the PLL */
    MC_ME.DRUN_MC.B.PLLON=1;
    /* Select PLL as sysclk for cores */
    MC_ME.DRUN_MC.B.SYSCLK=2; 						

    /* enter the drun mode, to update the configuration */
    /* Mode & Key */
    MC_ME.MCTL.R = 0x30005AF0;
    /* Mode & Key inverted */
    MC_ME.MCTL.R = 0x3000A50F;
    /* Wait for mode entry to complete */
    while(MC_ME.GS.B.S_MTRANS == 1);
    /* Check DRUN mode has been entered */
    while(MC_ME.GS.B.S_CURRENT_MODE != 0x3);
    /* waiting for PLL to lock */
    while(PLLDIG.PLLSR.B.LOCK ==0);					
}
/*=============================================================================
  FUNCTION: host_pll_firc_setup

  DESCRIPTION:
  This function configures the PLL clock for FIRC reference clock

  ARGUMENTS PASSED:
  None
  
  RETURN VALUE:
  None
  
  PRE-CONDITIONS:
  None

  POST-CONDITIONS:
  None

  IMPORTANT NOTES:
  None
  ===========================================================================*/

void host_pll_firc_setup()
{
    /* select FIRC as source for PLL(16MHz)
    MC_CGM.AC5_SC.B.SELCTL = 0; 					

    /* Configure PLL Dividers - 160MHz from 16Mhz FIRC */
    /* divide by 8 */
    PLLDIG.PLLDV.B.RFDPHI1 = 2;
    /* divide by 4 */
    PLLDIG.PLLDV.B.RFDPHI = 1;
    /* divide by 1 */     
    PLLDIG.PLLDV.B.PREDIV = 1;
    /* divide by 40 */
    PLLDIG.PLLDV.B.MFD = 40;

    /* fractional loop division factor = 0 */
    PLLDIG.PLLFD.B.MFN = 0;

    /* Loss-of-lock flag */	
    PLLDIG.PLLSR.B.LOLF = 1;
    /* loss of lock interrupt enable */
    PLLDIG.PLLCR.B.LOLIE = 1;
    /* denominator of fractional loop division factor */
    PLLDIG.PLLCAL3.R = 0x09C3C000;
    /* sigma delta modulation disabled */
    PLLDIG.PLLFD.B.SMDEN = 1;

    /* FIRC on */
    MC_ME.DRUN_MC.B.FIRCON=1;
    /* Enable the PLL */
    MC_ME.DRUN_MC.B.PLLON=1;
    /* Select PLL as sysclk for cores */
    MC_ME.DRUN_MC.B.SYSCLK=2;

    /* enter the drun mode, to update the configuration */
    /* Mode & Key */
    MC_ME.MCTL.R = 0x30005AF0;
    /* Mode & Key inverted */
    MC_ME.MCTL.R = 0x3000A50F;
    /* Wait for mode entry to complete */
    while(MC_ME.GS.B.S_MTRANS == 1);
    /* Check DRUN mode has been entered */  
    while(MC_ME.GS.B.S_CURRENT_MODE != 0x3); 
    /* waiting for PLL to lock */ 
    while(PLLDIG.PLLSR.B.LOCK ==0);
}

